/*
 *  rti860.c
 *
 *  LINUX device driver for RTI860 hardware
 *
 *  Copyright (C) 1995 Ralf Haller
 *  History: 11.05.95 birth
 *
 */

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/malloc.h>
#include <linux/fcntl.h>
#include <linux/timer.h>
#include <linux/delay.h>

#include <asm/segment.h>
#include <asm/io.h>
#include <asm/system.h>

#include "rti860.h"

/* major device number for rti860 device */
#define RTI860_MAJOR 193
/* IMPORTANT: check MAX_CHRDEV in major.h (MAX_CHRDEV >= RTI860_MAJOR) */

/* maximum number of minors */
#define RTI860_NO 24
#define RTI860_MAX 5

/* size of internal memory (256k) */
#define	RTI860_MEMSIZE 0x00040000L
#define RTI860_MEMCHUNK RTI860_MEMSIZE / 32

/* minimal and maximal sample frequency */
#define RTI860_MAX_FREQ 200e3
#define RTI860_MIN_FREQ 0.07

#define INDEX(a) (minor_index[a])

/* interval for polling mode (wait for buffer being half filled) */
static unsigned sleep_time = RTI860_MEMCHUNK * HZ / RTI860_MAX_FREQ;

/* delay for hardware im ms */
#define delay(a) __delay(a*loops_per_msec)

/*
 *  hardware registers
 */
#define status_reg          ((rti860[INDEX(minor)].base) + 0x00)
#define control_reg         ((rti860[INDEX(minor)].base) + 0x02)
#define muxdata_reg         ((rti860[INDEX(minor)].base) + 0x04)
#define softcon_reg         ((rti860[INDEX(minor)].base) + 0x06)
#define adcdata_reg         ((rti860[INDEX(minor)].base) + 0x08)
#define adcmem_reg          ((rti860[INDEX(minor)].base) + 0x0A)
#define datamem_reg         ((rti860[INDEX(minor)].base) + 0x0C)
#define hostmem_reg         ((rti860[INDEX(minor)].base) + 0x0E)
#define adchstmem_reg       ((rti860[INDEX(minor)].base) + 0x10)
#define muxmemscan_reg      ((rti860[INDEX(minor)].base) + 0x12)
#define data9513_reg        ((rti860[INDEX(minor)].base) + 0x14)
#define ctrl9513_reg        ((rti860[INDEX(minor)].base) + 0x16)
#define flgsclr_reg         ((rti860[INDEX(minor)].base) + 0x18)
#define trigger_reg         ((rti860[INDEX(minor)].base) + 0x1A)
#define indaddr_reg         ((rti860[INDEX(minor)].base) + 0x1C)

/*
 * NB. we must include the kernel idenfication string to install the module.
 */
#include <linux/version.h>
static char kernel_version[] = UTS_RELEASE;

/* parameters for all possible rti860s, can be changed with ioctl */
static Rti860 rti860[RTI860_MAX] = 
{{0,0L,Channel1_4,Free,0,100000,0,5000,-1,{0L,0L},{5L,0L}},	/* default */
 {0,0L,Channel1_4,Free,0,100000,0,5000,-1,{0L,0L},{5L,0L}},
 {0,0L,Channel1_4,Free,0,100000,0,5000,-1,{0L,0L},{5L,0L}},
 {0,0L,Channel1_4,Free,0,100000,0,5000,-1,{0L,0L},{5L,0L}},
 {0,0L,Channel1_4,Free,0,100000,0,5000,-1,{0L,0L},{5L,0L}}};

static void rti860_timer( unsigned long );

/* timer for polling mode */
static struct timer_list timer[RTI860_MAX] =
{{NULL,NULL,0,0,rti860_timer},
 {NULL,NULL,0,1,rti860_timer},
 {NULL,NULL,0,2,rti860_timer},
 {NULL,NULL,0,3,rti860_timer}};

/* index to rti860-struct */
static int minor_index[RTI860_NO]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

/* wait queue for rti860 device driver */
static struct wait_queue *rti860_wait_q;

/* irq counts */
static unsigned short irq_count[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

/* number of delay loops per msec */
static int loops_per_msec;

/*****
  Der folgende Typ beschreibt ein Counter Mode Register des 9513. Die
  Bits sind in dieser merkwrdigen Reihenfolge, weil der 80x86 die
  lo-Bytes bei niedrigen Adressen abspeichert.
*****/
typedef struct s_CounterMode {
  unsigned output:3;
  unsigned increm:1;
  unsigned bcdCount:1;
  unsigned repeat:1;
  unsigned reloadFrom:1;
  unsigned specialGate:1;
  unsigned source:4;
  unsigned sourceEdge:1;
  unsigned gate:3;
} CounterMode;
#define cm2us(x) (* (unsigned short*)&(x) )

#ifdef DEBUG
/****************************************************************************
 * read one counter of the 9513
 ****************************************************************************/
static unsigned read_counter( unsigned int minor, unsigned int num)
{
  if( num>0 && num<6 ) {

    /***** 9513 in 16-bit mode */
    outw(0xFFEF, ctrl9513_reg); delay(1);

    /***** Copy to hold register */
    outw(0xFFA0+(1<<(num-1)), ctrl9513_reg); delay(1);

    /***** Select the counter */
    outw(0xFF00+num, ctrl9513_reg); delay(1);
  
    /***** Read the counter */
    (void)inw(data9513_reg); delay(1);	/* mode register */
    (void)inw(data9513_reg); delay(1);	/* load register */

    return inw(data9513_reg); 		/* hold register */
  }

  return 0;
}
#endif

/****************************************************************************
 * reset the 9513
 ****************************************************************************/
static void reset9513( unsigned int minor )
{
  outw( 0x0000, control_reg );		/* Control-register zurcksetzen */
  delay(1);
  /**** 9513 zweimal zurcksetzen */
  outw( 0xFFFF, ctrl9513_reg ); delay(1);
  outb( 0xFF, ctrl9513_reg );  delay(1);

  /***** 
    Betriebsart setzen, d.h. Master-Mode Register adressieren und dann
    den Wert 0x3100 als zwei einzelne Bytes ausgeben. Der Mode 0x3100
    bedeutet:
    ---Bitpositionen---   ----eingestellte Funktion bei 0x3100----
    1000 0000 0000 0000 : Frequenzscalierung binr (nicht BCD)
    0100 0000 0000 0000 : automatisches Adreincrement
    0010 0000 0000 0000 : 16-bit Datenbus (ab sofort)
    0001 0000 0000 0000 : FOUT Aus
    0000 1111 0000 0000 : FOUT Divider = Division durch 1
    0000 0000 1111 0000 : FOUT-Quelle ist E1
    0000 0000 0000 1100 : Comparator 1 und 2 aus
    0000 0000 0000 0000 : Keine Funktion als Uhr (time-of-day) 
  *****/
  outb( 0x17, ctrl9513_reg ); delay(1);
  outb( 0x00, data9513_reg ); delay(1);
  outb( 0x31, data9513_reg ); delay(1);
  
  /*****
    Zhler 1 wird initialisiert:
    0x0005 heit: kein Gating, zhle steigende Flanke, zhle Ausgang
    von Zhler 0 (?), kein special gate, nachladen vom Loadregister,
    einmal zhlen, binr zhlen, abwrts zhlen, active low terminal
    count pulse
  *****/
  outw( 0xFF01, ctrl9513_reg ); delay(1);
  outw( 0x0005, data9513_reg ); delay(1);

  /* Eine A/D-Wandlung auslsen (warum?) */
  outw( 0x1234, softcon_reg ); delay(1);

  /*****
    Zhler 1 erneut initialisieren. (warum?)
    0x0002 wie oben, auer Ausgabe auf 'terminal count toggled'.
    Anschlieend wird der Ausgabepin definiert auf 0 gesetzt.
  *****/
  outw( 0xFF01, ctrl9513_reg ); delay(1);    
  outw( 0x0002, data9513_reg ); delay(1);
  outw( 0xFFE1, ctrl9513_reg ); delay(1);
  
  /*****
    Zhler 3 identisch mit Zhler 1 initialisieren.
  *****/
  outw( 0xFF03, ctrl9513_reg ); delay(1);
  outw( 0x0002, data9513_reg ); delay(1);
  outw( 0xFFE3, ctrl9513_reg ); delay(1);
  
  /*****
    Zhler 5 identisch mit Zhler 1 initialisieren.
  *****/
  outw( 0xFF05, ctrl9513_reg ); delay(1);
  outw( 0x0002, data9513_reg ); delay(1);
  outw( 0xFFE5, ctrl9513_reg ); delay(1);

  /* Kontrollregister zurcksetzen */  
  outw( 0x0000, control_reg ); delay(1);
}

/****************************************************************************
 * set the trigger
 ****************************************************************************/
static void init_overflow_counter( unsigned int minor )
/*****
  Zhler 4 und 5 auf dem 9513 sind wohl dazu gedacht, die
  Speicherberlaufbits anzutreiben (vgl. RTI860-Handbuch S.4-10 und
  S.B-13). Beide werden hier initialisiert, so da nach der halben
  Speichergre jeweils ein positiver Puls am pin O5 des 9513
  ausgegeben wird. Zhler 4 lassen wir bis 4 zhlen. Damit treibt
  er Zhler 5, den wir jeweils bis 0x8000 zhlen lassen, so da insgesamt
  bis 2^17 = halbe Speichergre der RTI860 gezhlt wird.
*****/
{
  CounterMode mode;

  /***** Zhler 4 und 5 abschalten */
  outw(0xFFC0 + (1<<(5-1)) + (1<<(4-1)), ctrl9513_reg);

  /***** Zhler 4 */
  mode.gate = 0;
  mode.sourceEdge = 0;
  mode.source = 4;
  mode.specialGate = 0;
  mode.reloadFrom = 0;
  mode.repeat = 1;
  mode.bcdCount = 0;
  mode.increm = 0;
  mode.output = 1;
  outw( 0xFF04, ctrl9513_reg ); delay(1);
  outw( cm2us(mode), data9513_reg ); delay(1); /* Mode */
  outw( 0x0004, data9513_reg );  delay(1); /* Loadregister */
  outw( 0x0000, data9513_reg );      delay(1); /* Holdregister */
  outw( 0xFF48, ctrl9513_reg ); delay(1); /* Lade von Loadregister */

  /***** Zhler 5 wird von Zhler 4 betrieben */
  mode.gate = 0;
  mode.sourceEdge = 0;
  mode.source = 0;
  mode.specialGate = 0;
  mode.reloadFrom = 0;
  mode.repeat = 1;
  mode.bcdCount = 0;
  mode.increm = 0;
  mode.output = 1;
  outw( 0xFF05, ctrl9513_reg ); delay(1);
  outw( cm2us(mode), data9513_reg ); delay(1); /* Mode */
  outw( 0x8000, data9513_reg ); delay(1); /* Loadregister */
  outw( 0x0000, data9513_reg );      delay(1); /* Holdregister */
  outw( 0xFF50, ctrl9513_reg ); delay(1); /* Lade von Loadregister */
}

/****************************************************************************
 * set the trigger
 ****************************************************************************/
static void set_trigger( unsigned int minor )
{
  unsigned int mask;
  int level;

  switch ( rti860[INDEX(minor)].trigger ) {
  case Free:
    mask = 0x000c;
    break;
  case BereichPlus:
    mask = 0x0003;
    break;
  case BereichMinus:
    mask = 0x0002;
    break;
  case PegelPlus:
    mask = 0x0001;
    break;
  case PegelMinus:
    mask = 0x0000;
    break;
  case TTLTrigger:
    mask = 0x0008;
    break;
  default:
    printk("Wrong trigger in set_trigger, using no trigger!\n");
    mask = 0x000c;
    break;
  }
  
  level	= (int) (1.28 * rti860[INDEX(minor)].level);
  
  if ( level >  127 ) level = 127;
  else if ( level < -127 ) level = -127;
  
  mask |= (0x8000 + level * 128) & 0xFF00;
  
  outw( mask, trigger_reg );
  
}

/****************************************************************************
 * set the channels to scan
 ****************************************************************************/
static void set_inputmux( unsigned int minor )
{
  unsigned short I;
  unsigned base=8;

  outw( 0x0000, control_reg );

  switch( rti860[INDEX(minor)].channel ) {
  case Channel1_16:
    for ( I = 0; I < 16; I++ ) {
      outw( I, muxmemscan_reg );
      outw( 15-I, muxdata_reg );
    }
    break;
  case Channel1_8:
    base = 0;
  case Channel9_16:
    for( I=0; I<16; I++) {
      outw( I, muxmemscan_reg );
      outw( (15-I)%8 + base, muxdata_reg );
    }
    break;
  case Channel1_4:
  case Channel5_8:
  case Channel9_12:
  case Channel13_16:
    outw( 0x0000, control_reg );
    for ( I = 0; I < 16; I++ ) {
      outw( I, muxmemscan_reg );
      outw( ((15-I) & 0x03) + (rti860[INDEX(minor)].channel << 2), muxdata_reg);
    }
    break;
  default: 
    printk("Wrong mode in set_inputmux!\n");
  }
  outw( 0x0F, muxmemscan_reg );
  delay(1);
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
void init_clock( unsigned int minor )
/*****
  Zhler 1 des 9513 ist fr die Clock zustndig. Wenn eine externe
  Clock verwendet wird, wird der Ausgang des Zhlers lediglich in
  einen definierten Zustand gebracht. (Warum allerdings so aufwendig,
  wei icht nicht, Kir.)
*****/
{
  unsigned short mode, prescale, scale;

  if( rti860[INDEX(minor)].frequency == 0 ) {
    outw( 0xFF01, ctrl9513_reg );     /* Counter 1 anwhlen */
    delay(1);
    outw( 0x0602, data9513_reg );     /* Counter 1 auf TC toggled */
    delay(1);                         /* source gate 1, count once? */
    outw( 0x0003, data9513_reg );     /* Loadregister = 3 */  
    delay(1);
    outw( 0x0000, data9513_reg );     /* Holdregister = 0 */
    delay(1);                           
    
    outw( 0xFFE1, ctrl9513_reg );     /* Ausgang Counter 1 auf low */
    delay(1);                         /* => Karte gesperrt */
    outw( 0xFF41, ctrl9513_reg );     /* Load Counter 1 with Loadreg. */
    delay(1);
    outw( 0xFFF1, ctrl9513_reg );     /* Counter 1 steppen, 1. mal */
    delay(1);
    outw( 0xFFF1, ctrl9513_reg );     /* Counter 1 steppen, 2. mal */
    delay(1);
    outw( 0xFFF1, ctrl9513_reg );     /* Counter 1 steppen, 3. mal */
    delay(1);

  } else { /* Wenn interner Takt */
    /*****
     Die folgenden Zeilen initialisieren Counter 1. Counter 1
     erzeugt den Takt (den sog. Pacer) fr die A/D-Wandlung, wenn
     kein externer Takt verwendet wird.
     Der Modus=0x9025 + (Vorteiler+0x000B)<<8
     bedeutet (vgl. S.2-141 im Am9513-Handbuch):
          Gating Control: active hi gate 1      1110 0000 0000 0000
	  Source Edge:    Count on Falling Edge 0001 0000 0000 0000
	  Count Source:   F_Vorteiler           0000 1111 0000 0000
	  Special Gate:   disabled              0000 0000 1000 0000
	  Reload from:    Load Register         0000 0000 0100 0000
	  Repetitively:   TRUE                  0000 0000 0010 0000
	  Count Type:     binary                0000 0000 0001 0000
	  Direction:      count down            0000 0000 0000 1000
	  Output:         Active Lo Pulse       0000 0000 0000 0111
    *****/
    prescale = rti860[INDEX(minor)].prescale;
    scale = rti860[INDEX(minor)].scale;
    mode = 0x9025 | ( (prescale+0x000B)<<8 );
    outw( 0xFF01, ctrl9513_reg );	delay(1);
    outw( mode, data9513_reg );		delay(1);
    outw( scale, data9513_reg );	delay(1);             
    outw( 0x0000, data9513_reg );       delay(1); /* Hold Register = 0 */
    outw( 0xFF41, ctrl9513_reg );	delay(1); /* Lade mit Loadreg */
  }
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void init_posttrigger_counter( unsigned int minor )
/*****
  Zhler 2&3 zhlt A/D-Wandlungen nach dem Trigger. Sobald 2&3 auf 0
  zhlt, hrt die RTI860 mit A/D-Wandlungen auf. Deshalb werden 2&3
  in RTI860_Start() bei kontinuierlichem Modus nicht scharfgemacht
*****/
{
  /*****
    Der Modus=0x1221 bedeutet (vgl. S.2-141 im Am9513-Handbuch):
        Gating Control: No Gating             1110 0000 0000 0000
        Source Edge:    Count on Falling Edge 0001 0000 0000 0000
        Count Source:   SRC 2                 0000 1111 0000 0000
        Special Gate:   disabled              0000 0000 1000 0000
        Reload from:    Load Register         0000 0000 0100 0000
        Repetitively:   TRUE                  0000 0000 0010 0000
        Count Type:     binary                0000 0000 0001 0000
        Direction:      count down            0000 0000 0000 1000
        Output:         Active Hi Pulse       0000 0000 0000 0111
    Es stellt sich die Frage, warum das Loadregister gerade auf
    samples-1 gesetzt wird, wenn sowieso kontinuierlich gearbeitet
    wird.
  *****/
  outw( 0xFF02, ctrl9513_reg ); delay(1);  /* Adressieren */
  outw( 0x1221, data9513_reg ); delay(1);  /* Modus setzen */
  outw( 0, data9513_reg ); delay(1);  /* Loadregister */
  outw( 0x0000, data9513_reg ); delay(1);  /* Holdregister */
  outw( 0xFF42, ctrl9513_reg ); delay(1); /* Counter:=Loadregister */

  /*****
    Counter 3 zhlt zusammen mit Counter 2 die gewandelten Daten. Der
    Output von Counter 3 ist sogar am Ausgabeport der RTI860 zu finden
    --- als DONE-Signal. Auderdem hlt die Karte tatschlich an,
    wenn der Ausgang von Counter 3 auf lo geht.
  *****/
  outw( 0xFF03, ctrl9513_reg );    /* Source: Counter 3 - single count */
  delay(1);                         /* TC toggle */ 
  outw( 0x0002, data9513_reg );    /* Source: Counter 2 */
  delay(1);                         /* single count, TC toggle */
  outw( 0x0003, data9513_reg );    /* Loadregister = 3 */
  delay(1);
  outw( 0x0000, data9513_reg );    /* Holdregister = 0 */
  delay(1);
  outw( 0xFF44, ctrl9513_reg );    /* Load Counter 3 with Loadregister*/
  delay(1);
  outw( 0xFFF3, ctrl9513_reg );    /* Counter 3 steppen erstes mal */
  delay(1);
  outw( 0xFFF3, ctrl9513_reg );    /* zweites mal */
  delay(1);
  outw( 0xFFF3, ctrl9513_reg );    /* drittes mal */
  delay(1);
  outw( 0xFFEB, ctrl9513_reg );    /* Ausgang Counter 3 auf high */
  delay(1);
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_clearmem( unsigned int minor )
{
  int i;
  /***** Setze ADC Zieladresse auf 0 */
  outw( 0x8100, adchstmem_reg );	/* most sign. bits */
  outw( 0x8100, adchstmem_reg );
  outw( 0x1000-20, adcmem_reg );/* least sign. bits */
  for(i=0; i<40; i++) outw( 0, control_reg );

  outw( 0x8100, adchstmem_reg );	/* most sign. bits */
  outw( 0x8100, adchstmem_reg );
  outw( 0x0000, adcmem_reg );/* least sign. bits */

  /***** Setze Leseadresse auf 0 */
  outw( 0x0001, adchstmem_reg );	/* most sign. bits */
  outw( 0x0001, adchstmem_reg );
  outw( 0x0000, hostmem_reg );	/* least sign. bits */
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_reset( unsigned int minor )
/*****
  Beended die A/D-Wandlung auf der Karte. Der Speicher der Karte wird
  gelscht. Die Karte kann ohne Neuinitialisierung wieder gestartet
  werden. 
*****/
{
  reset9513( minor ); delay(5);
  rti860_clearmem( minor );
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_init( unsigned int minor )
{

  rti860_reset(minor);

  set_inputmux(minor);
  set_trigger(minor);

}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_start( unsigned int minor )
/*****
  Startet die A/D-Wandlung auf der Karte mit den in RTI860_Init
  vorgegebenen Parametern. Die A/D-Daten werden im Speicher der Karte
  abgelegt. Wurde Betriebsart=Fix gewhlt, so beendet die Karte
  selbstndig die Messung nach der vorgegebenen Anzahl der
  Einzelmessungen. Andernfalls mu die Karte mit RTI860_Stop oder
  RTI_860_Reset angehalten werden.
  Hinweis: Diese Prozedur setzt den Speicher der Karte nicht zurck.
*****/
{
  unsigned short control;

  reset9513(minor);

  /***** clear the overflow flags of the 9513 */
  outw_p(0x0000, flgsclr_reg); delay(1);
  outw_p(0x8000, flgsclr_reg); delay(1);

  /***** Steuerwort der RTI860 setzen: */
  control = (1<<1)	/* Memory Mode Enable */
          | (1<<2)     	/* Scan Mode Enable */
          | (1<<8)	/* Simultaneous Mode Enable */
          | (1<<5);    	/* Continuous Transfer Mode Enable */

  if( rti860[INDEX(minor)].irq >= 0 )
    control |= (1<<14);	/* continuous transfer interrupt */

  outw_p( control, control_reg); delay(1);

  init_clock(minor);
  init_posttrigger_counter(minor);
  init_overflow_counter(minor);

  /***** Zhler 1,4,5 scharfmachen */
  outw_p( 0xFF39, ctrl9513_reg );

  /***** Wandlung starten */
  outw_p( control|1, control_reg);
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_stop( unsigned int minor )
/*****
  Beendet die A/D-Wandlungen auf der Karte. Der Speicher der Karte
  wird *nicht* gelscht. Die Karte kann ohne Neuinitialisierung wieder
  gestartet werden.
*****/
{
  reset9513( minor );
}

#ifdef DEBUG
/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static unsigned rti860_get_readaddress( unsigned int minor )
{
  return
    ((unsigned)(inw(adchstmem_reg) & 0x0003) << 16)
    + inw_p(hostmem_reg);
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static unsigned rti860_get_writeaddress( unsigned int minor)
{
  unsigned short hiBits, hiTmp;
  unsigned writeAdr;
  /*****
    Zu der folgenden Konstruktion siehe den Kommentar in Fuellstand.
  *****/
  do {
    hiBits = inw(adchstmem_reg) & 0x0300;
    writeAdr = inw(adcmem_reg);
    hiTmp = inw(adchstmem_reg) & 0x0300;
  } while( hiBits!=hiTmp );
  writeAdr +=  ( (unsigned)hiBits << 8 );

  return writeAdr;
}
#endif

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static unsigned rti860_get_filling( unsigned int minor )
{
  unsigned writeAdr, readAdr;
  unsigned short hiBits, hiTmp;

  /***** 
    Das Problem beim Auslesen der Schreibadresse ist, da zwei
    Speicherzellen aus der Karte gelesen werden mssen, und es ist
    nicht ausgeschlossen, da die Karte die Schreibadresse zwischen
    den beiden Zugriffen erhht. Wenn diese Erhhung ber die
    0xFFFF-Marke hinausgeht, passen die beiden gelesenen Werte nicht
    zueinander, egal in welcher Reihenfolge man sie einliest. Deshalb
    die folgende Konstruktion:
  *****/
  do {
    hiBits = inw(adchstmem_reg) & 0x0300;
    writeAdr = inw(adcmem_reg);
    hiTmp = inw(adchstmem_reg) & 0x0300;
  } while( hiBits!=hiTmp );
  writeAdr +=  ( (unsigned)hiBits << 8 );

  hiBits = inw(adchstmem_reg) & 0x0003;
  readAdr = inw(hostmem_reg) + ( (unsigned)hiBits << 16 );
  
  if( writeAdr>=readAdr ){
    return writeAdr - readAdr;
  } else {
    return writeAdr + RTI860_MEMSIZE - readAdr;
  }
}

/****************************************************************************
 * interrupt service routine
 ****************************************************************************/
static void rti860_interrupt( int irq, struct pt_regs *regs )
{
  /*printk("irq%d\n",irq);*/
  /* wake us up */
  wake_up(&rti860_wait_q);
}

/****************************************************************************
 * timer service routine
 ****************************************************************************/
static void rti860_timer( unsigned long data )
{
  /*printk("timer%ld\n",data);*/
  /* wake us up */
  wake_up(&rti860_wait_q);
}

/****************************************************************************
 * allocate the interrupt
 ****************************************************************************/
static void rti860_alloc_irq( int minor )
{
  int ret;

  /* irq already allocated from driver ? */
  if( irq_count[rti860[INDEX(minor)].irq] == 0 ) {
      
    /* get the interrupt */
    ret = request_irq( rti860[INDEX(minor)].irq, rti860_interrupt, SA_INTERRUPT, "rti860");
    if( ret ) {
      /* no interrupt */
      printk("cannot allocate irq%d, switching to polling mode...\n",rti860[INDEX(minor)].irq);
      rti860[INDEX(minor)].flags &= ~RTI860_F_IRQ;
      return;
    }
    else {
      /* got interrupt */
      printk("allocated irq%d\n",rti860[INDEX(minor)].irq);
    }
  }

  /* increase irq count */
  irq_count[rti860[INDEX(minor)].irq]++;

  /* set flags */
  rti860[INDEX(minor)].flags |= RTI860_F_IRQ;

}

/****************************************************************************
 * free the interrupt
 ****************************************************************************/
static void rti860_free_irq( int minor )
{
  /* interrupt to free ? */
  if( rti860[INDEX(minor)].flags & RTI860_F_IRQ ) {

    /* decrease irq_count */
    if( irq_count[rti860[INDEX(minor)].irq] == 0 ) {
      printk("rti860: irq_count less than zero! setting back...\n");
      irq_count[rti860[INDEX(minor)].irq] = 0;
    }

    if( --irq_count[rti860[INDEX(minor)].irq] == 0 ) {
      /* free interrupt and reset flags */
      free_irq( rti860[INDEX(minor)].irq );
      printk("freed irq%d\n",rti860[INDEX(minor)].irq);
    }

    /* reset flags */
    rti860[INDEX(minor)].flags &= ~RTI860_F_IRQ;

  }
}

/****************************************************************************
 * read data from hardware
 ****************************************************************************/
static int rti860_read(struct inode *inode, struct file *file, char *buf,
		       int count)
{
  int ret, pre, post;
  int index=0, tout=0;
  unsigned num_samples = 0;
  unsigned int minor = MINOR(inode->i_rdev);

  /* only even count allowed */
  count &= 0xFFFE;

  /* check user memory */
  ret = verify_area( VERIFY_WRITE, buf, count );
  if (ret) return (ret);

  /* device busy ? */
  if( (rti860[INDEX(minor)].flags & RTI860_F_BUSY) == 0 ) {

    /* do we need interrupt ? */
    if( rti860[INDEX(minor)].irq >= 0 ) {

      /* get irq */
      rti860_alloc_irq(minor);
    }

    /* reset and start the hardware */
    rti860_init(minor);
    rti860_start(minor);

    /* set device busy */
    rti860[INDEX(minor)].flags |= RTI860_F_BUSY;

  }

  num_samples = rti860_get_filling(minor);

  /* until all data is read */
  while( index < count ) {

    /* test for overflow */
    if( inw(status_reg) & (1<<6) ) {

      /* reset mode ? */
      if( rti860[INDEX(minor)].flags & RTI860_F_RESET ) {

	/* start again */
	rti860_init(minor);
	rti860_start(minor);
      }
      else
	/* return data already read or error if nothing read */
	return index ? index : -EIO;
    }

    /* do we have to wait */
    while ( num_samples == 0 ) {

      /* non blocked ? */
      if( file->f_flags & O_NONBLOCK )
	/* return from read */
	return index;

      /* polling mode */
      if((rti860[INDEX(minor)].flags & RTI860_F_IRQ) == 0 ){

	/* check if timer already set */
	if( timer[INDEX(minor)].next == NULL && timer[INDEX(minor)].prev == NULL ) {

	  /* if not set timer */
	  if( (count-index)/sizeof(short) >= RTI860_MEMCHUNK )
	    timer[INDEX(minor)].expires = sleep_time;
	  else
	    timer[INDEX(minor)].expires = (count-index) /
	      sizeof(short) * HZ / 4 / rti860[INDEX(minor)].frequency / 1000;

	  add_timer( &(timer[INDEX(minor)]) );
	}
      }

      /* timeout set ? */
      if( !tout && 
	  (rti860[INDEX(minor)].timeout.tv_sec || 
	   rti860[INDEX(minor)].timeout.tv_usec)) {
	
	/* set timeout */
	current->timeout = jiffies + 
	  rti860[INDEX(minor)].timeout.tv_sec * HZ +           /* seconds */
	  rti860[INDEX(minor)].timeout.tv_usec * HZ / 1000000; /* microseconds */
	
	/* set timeout marker */
	tout = 1;
      }
      
      /* fall asleep until interrupt or timer arrives */
      interruptible_sleep_on(&rti860_wait_q);
      
      if (current->signal & ~current->blocked) {
	return index;
      }
    
      /* timeout occured ? */
      if( (current->timeout == 0) && tout )
	return (-ETIME);

      num_samples = rti860_get_filling(minor);

    }

    /* reset timeout marker */
    tout=0;

    if( num_samples > (count-index)/sizeof(short) )
      num_samples = (count-index)/sizeof(short);

    pre = inw(adchstmem_reg)&0x2;
#ifdef DEBUG
    printk("%u %u %u %u\n",num_samples,
	   rti860_get_filling(minor),
	   rti860_get_readaddress(minor),
	   rti860_get_writeaddress(minor));
#endif
    for( ; num_samples > 0 ; num_samples-- ) {

      /* read data and copy to user space */
      /* division by 16 because the 4 lower bits are not used */ 
      put_fs_word( (short)inw_p(datamem_reg)/16, &(buf[index]));
      index = index + sizeof(short);
    }

    num_samples = rti860_get_filling(minor);

    post = inw(adchstmem_reg)&0x2;
    if( pre != post ) {
#if 0
      printk("cleared CXFLAG %u %u\n",rti860_get_readaddress(minor),
	                              rti860_get_writeaddress(minor));
#endif
      outw( 0x8000, flgsclr_reg );
    }
  }

  return index;
}

/****************************************************************************
 * test if data can be read
 ****************************************************************************/
static int rti860_select(struct inode *inode, struct file *file, 
			 int sel_type, select_table *wait)
{
  unsigned int minor = MINOR(inode->i_rdev);

  /* ready for read ? */
  if( sel_type == SEL_IN ) {

    /* data available ? */
    if( rti860_get_filling(minor) == 0 ) {

      /* insert in wait queue */
      select_wait(&rti860_wait_q,wait);

      /* nothing to read */
      return 0;
    }

    /* ready to read */
    return 1;
  }

  /* always ready for write and exceptions */
  return 1;
}

/****************************************************************************
 * check the rti860-data
 ****************************************************************************/
static int rti860_check(Rti860 *rti860)
{

  if( rti860->frequency != 0 && 
      (rti860->frequency/1000 > RTI860_MAX_FREQ || rti860->frequency/1000 < RTI860_MIN_FREQ) )
    return 1;

  if( rti860->level < -100 || rti860->level > 100 )
    return 1;

  if( rti860->timeout.tv_sec<0 || rti860->timeout.tv_usec<0 ) 
    return 1;

  return 0;
}

/****************************************************************************
 * control the device
 ****************************************************************************/
static int rti860_ioctl(struct inode *inode, struct file *file,
			unsigned int cmd, unsigned long arg)
{
  unsigned int minor = MINOR(inode->i_rdev);
  Rti860 tmprti860;
  void *ptr = (void *)arg;
  int error;
  float scale;
  unsigned short prescale;

  switch( cmd ) {

  case RTI860_SET_PAR: /* set new parameters */

    /* device busy ? */
    if( rti860[INDEX(minor)].flags & RTI860_F_BUSY ) return (-EBUSY);

    /* check memory */
    error = verify_area( VERIFY_READ, ptr, sizeof(Rti860) );
    if (error)
      return (error);

    /* copy data */
    memcpy_fromfs( (void *)&tmprti860, ptr, sizeof(Rti860) );

    if( tmprti860.frequency != 0 ) {
      
      /* adjust sample frequency */
      switch( tmprti860.channel ) {
      case Channel1_16:
	tmprti860.frequency *= 4;
	break;
      case Channel1_8:
      case Channel9_16:      
	tmprti860.frequency *= 2;
	break;
      case Channel1_4:
      case Channel5_8:
      case Channel9_12:
      case Channel13_16:
	break;
      default:
	return (-EINVAL);
      }

      /* calculate prescale and scale factors for the frequency */
      for(scale = 5e6 * 1000 / tmprti860.frequency, prescale = 0;
	  scale > 65535.0;
	  prescale++, scale /= 16.0 ) /* just run */;
      tmprti860.prescale = prescale;
      tmprti860.scale = (unsigned short)scale;
    }

    /* check data */
    if( rti860_check( &tmprti860 ) )
      return (-EINVAL);
    
    /* set data */
    rti860[INDEX(minor)].flags = (rti860[INDEX(minor)].flags & ~RTI860_F_USER) | 
                                 (tmprti860.flags & RTI860_F_USER);
    rti860[INDEX(minor)].channel	= tmprti860.channel;
    rti860[INDEX(minor)].trigger        = tmprti860.trigger;
    rti860[INDEX(minor)].level          = tmprti860.level;
    rti860[INDEX(minor)].frequency      = tmprti860.frequency;
    rti860[INDEX(minor)].prescale       = tmprti860.prescale;
    rti860[INDEX(minor)].scale          = tmprti860.scale;
    rti860[INDEX(minor)].irq		= tmprti860.irq;
    rti860[INDEX(minor)].timeout	= tmprti860.timeout;

    if( tmprti860.frequency != 0 )
      sleep_time = RTI860_MEMCHUNK * HZ / tmprti860.frequency / 1000;
    else
      sleep_time = RTI860_MEMCHUNK * HZ / RTI860_MAX_FREQ;

    /* oK */
    return 0;  

  case RTI860_GET_PAR: /* get actual parameters */

    /* check memory */
    error = verify_area( VERIFY_WRITE, ptr, sizeof(Rti860) );
    if (error)
      return (error);

    /* copy data */
    memcpy_tofs( ptr, (void *)&rti860[INDEX(minor)], sizeof(Rti860) );

    /* oK */
    return 0;

  case RTI860_RESET: /* reset device */

    /* hardware reset */
    rti860_init(minor);
    
    /* set device not busy */
    rti860[INDEX(minor)].flags &= ~RTI860_F_BUSY;

    /* oK */
    return 0;

  default:
    return (-EINVAL);
  }
}

/****************************************************************************
 * open the device
 ****************************************************************************/
static int rti860_open(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);

  /* check minor device number */
  if( minor >= RTI860_NO )
    return (-ENODEV);

  /* check if hardware present */
  if( !(rti860[INDEX(minor)].flags & RTI860_F_PRESENT) )
    return (-ENODEV);

  /* open for write ? */
  if( (file->f_flags & O_ACCMODE) != O_RDONLY )
    return (-EROFS);

  /* already opened ? */
  if( rti860[INDEX(minor)].flags & RTI860_F_OPEN )
    return (-EBUSY);

  /* set device open */
  rti860[INDEX(minor)].flags |= RTI860_F_OPEN;

  /* oK */
  return 0;
}

/****************************************************************************
 * close the device
 ****************************************************************************/
static void rti860_release(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);

  /* clear irq */
  rti860_free_irq(minor);

  /* set device not busy */
  rti860[INDEX(minor)].flags &= ~RTI860_F_BUSY;

  /* set device closed */
  rti860[INDEX(minor)].flags &= ~RTI860_F_OPEN;
}

static struct file_operations rti860_fops = {
        NULL,		/* rti860_lseek */
        rti860_read,
        NULL,		/* rti860_write */
        NULL,		/* rti860_readdir */
        rti860_select,
        rti860_ioctl,
        NULL,		/* rti860_mmap */
        rti860_open,
        rti860_release
};

/****************************************************************************
 * initialize the device
 ****************************************************************************/
int init_module( void )
{
  unsigned long address;
  unsigned int minor, num=1;

  if( register_chrdev(RTI860_MAJOR, "rti860", &rti860_fops) )
  {
    printk("unable to get major %d for RTI860-device\n", RTI860_MAJOR);
    return -EIO;
  }

  /* find the rti860 */
  for( minor=0,address=0x100 ; address<=0x3E0 ; address+=0x20,minor++ ) {

    /* check if rti860 is present at address */
    if( (inw(address) & 0xF000) == 0x2000 ) {

      printk("rti860 found at 0x%lX\n", address );

      INDEX(minor) = num++;

      /*
       * Set present flag and base address
       */
      rti860[INDEX(minor)].flags |= RTI860_F_PRESENT;
      rti860[INDEX(minor)].base = address;

      /* maximal number of rti860's reached */
      if( num >= RTI860_MAX ) break;
    }
  }

  /* calculate delay loops */
  loops_per_msec = loops_per_sec/1000;

  printk("RTI860-device installed...\n");

  return 0;  

}

/****************************************************************************
 * kill the device
 ****************************************************************************/
void cleanup_module( void) {

  if (unregister_chrdev(RTI860_MAJOR, "rti860") != 0) {
    printk("cleanup_module failed\n");
  }
  else {
    printk("cleanup_module succeeded\n");
  }    
}
