/*****************************************************************
  Setup program for for RTI860 Linux device driver
  
  Copyright (C) 1995 Ralf Haller
            (C) 1995, 1996 Harald Kirsch (kir@iitb.fhg.de)

	    Fraunhofer Institut fr 
	    Informations- und Datenverarbeitung (IITB)
	    Fraunhoferstr. 1
	    76131 Karlsruhe

*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "rti860.h"

#include "cmdline.h"
/*****************************************************************/

#define TRUE  (0<1)
#define FALSE (1<0)

static int irqs[] = {0,3,5,7,10,11,12,15,-1};

static char *channels[]={
  "16","15","14","13","12","11","10","9","8","7","6","5","4","3","2","1",
  "1-4","5-8","9-12","13-16","1-16","1-8","9-16",NULL
};
/* static int num_channels[]={4,4,4,4,16,8,8};*/
static char *triggers[]={
  "free", 
  "+edge", 
  "-edge", 
  "+level", 
  "-level",
  "TTL",
  NULL
};

/*****************************************************************/
static int
transferValues(Rti860 *rti860, Cmdline *cmd)
{
  int haveError = 0, res = 0;
  int i;

  /***** interrupt channel */
  if( cmd->irqP ) {
    for(i=0; irqs[i]>=0 && cmd->irq!=irqs[i]; i++) /* just run */;
    if( irqs[i]==-1 ) {
      fprintf(stderr, 
	      "%s: wrong irq, only 0, 3, 5, 7, 10, 12 or 15 allowed\n",
	      Program);
      haveError = 1;
    } else {
      res = 1;
      rti860->irq = cmd->irq;
    }
  }

  /****** restart bit */
  if( cmd->restartP ) {
    res = 1;
    if( cmd->restart ) rti860->flags |= RTI860_F_RESTART;
    else rti860->flags &= ~RTI860_F_RESTART;
  }

  /***** channel(s) */
  if( cmd->channelP ) {
    for(i=0; channels[i] && (0!=strcmp(channels[i],cmd->channel)); i++)
      /* just run */;
    if( !channels[i] ) {
      fprintf(stderr, 
	      "%s: Wrong channel, use 1,..,16, 1-4, 5-8, 9-12, "
	      "13-16, 1-8, 9-16, 1-16\n", Program);
      haveError = 1;
    } else {
      res = 1;
      rti860->channel = i-16;
    }
  }

  /***** trigger */
  if( cmd->triggerP ) {
    for(i=0; triggers[i] && (0!=strcmp(triggers[i],cmd->trigger)); i++)
      /* just run */ ;
    if( !triggers[i] ) {
      fprintf(stderr, 
	      "%s: Wrong trigger, use free, +edge, -edge, "
	      "+level, -level, TTL\n", Program);
      haveError = 1;
    } else {
      res = 1;
      rti860->trigger = i;
    }
  }

  /***** trigger-level */
  if( cmd->triggerLevelP ) {
    rti860->level = cmd->triggerLevel;
    res = 1;
  }

  /***** frequency or divider */
  if( cmd->frequency && cmd->dividerP ) {
    fprintf(stderr, 
	    "%s: cannot use frequency (-f) and divider (-d) "
	    "at the same time\n", Program);
    haveError = 1;
  }

  if( cmd->frequencyP ) {
    rti860->tics = 5*1000*1000/cmd->frequency;
    res = 1;
  }

  if( cmd->dividerP ) {
    if( cmd->divider==1 ) {
      fprintf(stderr, 
      "\n%s: An external clock divider of `1' is not supported due to\n"
      "  hardware limitations. But you may want to try the EXTERNAL\n"
      "  PACER (Pin 40) instead of EXTERNAL CLOCK (Pin 41), because\n"
      "  the divider is not used then. However, the trigger-logic\n"
      "  does not work then.", Program);
      haveError = 1;
    } else {
      rti860->tics = -cmd->divider;
      res = 1;
    }
  }

  /***** timeout */
  if( cmd->timeoutP ) {
    rti860->timeout = cmd->timeout;
    res = 1;
  }

  /***** sleep time */
  if( cmd->pollintP ){
    rti860->sleep_time = cmd->pollint;
    res = 1;
  }

  if( haveError ) exit(1);
  return res;
}
/*****************************************************************/
static void
showValues(char *device, Rti860 *rti860)
{
  printf("Current settings for `%s' at address 0x%x:\n",
	 device, rti860->base);

  /***** interrupt channel or polling interval */
  if( rti860->irq ) {
    printf("  interrupt number:             %d\n", rti860->irq);
  } else {
    printf("  polling interval:             %d jiffies (%dms)\n", 
           rti860->sleep_time, rti860->sleep_time*10);
  }

  /***** timeout */
  printf("  timeout for read operations:  %d seconds\n",
         rti860->timeout);

  /***** channels currently active */
  printf("  sampled channel(s):           ");
  if( rti860->channel<0 ) printf("%d\n", -rti860->channel);
  else printf("%s\n", channels[rti860->channel+16]);

  printf("  restart bit:                  %s\n",
         rti860->flags&RTI860_F_RESTART ? "on" : "off");

  /***** triggering */
  printf("  trigger type:                 %s\n", 
	 triggers[rti860->trigger]);
  if( rti860->trigger>=1 && rti860->trigger<=4 ) {
    printf("  trigger level:                %d\n", rti860->level);
  }

  /***** sampling frequency or external divider */
  if( rti860->tics>0 ) {
    unsigned prescale, scale;
    double f;
    int div = 0;

    for(prescale=1,scale=rti860->tics;  
	scale > 65535;	  
	prescale*=16,scale/=16);
    f = 200e-9*(double)prescale*(double)scale;
    printf("  sampling frequency (Hz)       %f\n", 1.0/f);

    if( rti860->channel==Channel1_16 ) div=4;
    else if( rti860->channel==Channel1_8 
	     || rti860->channel==Channel9_16 ) div=2;
    if( div ) {
      fprintf(stderr,
     "    REMARK: Due to the channel selection (%d blocks of 4) the\n"
     "    driver will scale this sampling frequency internally by %d.\n",
	      div, div);
    }
  } else {
    int div = 0;
    printf("  external clock divider:       %d\n", -rti860->tics);
    
    if( rti860->channel==Channel1_16 ) div=4;
    else if(  rti860->channel==Channel1_8 
	      || rti860->channel==Channel9_16 ) div=2;
    if( div ) {
      fprintf(stderr,
      "    WARNING: Due to the channel selection (%d blocks of 4) the\n"
      "    external clock is effectively divided by %d (not %d).\n", 
	      div, -rti860->tics*div, -rti860->tics);
    }
  }
}
/*****************************************************************/
int 
main( int argc, char **argv )
{
  int device;
  Rti860 rti860;
  Cmdline *cmd = parseCmdline(argc, argv);
  int errPos = 1;
  
  /***** open the device */
  device = open(cmd->argv[0], O_RDONLY);
  if( device==-1 ) {
    fprintf(stderr, 
            "%s: %s: %s\n", Program, cmd->argv[0], strerror(errno));
    return 1;
  }

  /***** get current settings */
  if( -1==ioctl(device, RTI860_GET_PAR, &rti860) ) {
  unexErr:
    fprintf(stderr,
            "%s(%d): unexpected error `%s'\n"
            "while accessing `%s'. "
            "Are you sure this is an rti860 device?\n",
            Program, errPos, strerror(errno), cmd->argv[0]);
    return 1;
  }

  /***** Transfer values to the driver, if any option was set */
  if( transferValues(&rti860, cmd) ) {
    /* printf("Wishlist:");
    showValues(cmd->argv[0], &rti860); */

    /***** write to driver */
    errPos += 1;
    if( -1==ioctl(device, RTI860_SET_PAR, &rti860) ) goto unexErr;
    
    /***** read from driver */
    errPos += 1;
    if( -1==ioctl(device, RTI860_GET_PAR, &rti860) ) goto unexErr;
  }
  
  /***** Print the result.*/
  if( !cmd->silentP ) {
    showValues(cmd->argv[0], &rti860);
  }
  return 0;
}
